#!/bin/bash

source ./a6e_azure_create_hubdps.conf

function check_config() {
    if [ -z "${1}" ]; then
        echo "Please input ${2} to config."
        exit
    fi
}

check_config "${resourceGroup}" "resourceGroup"
check_config "${iotHubName}" "iotHubName"
check_config "${provisioningServiceName}" "provisioningServiceName"

if [ ! -e "${certificateFilePath}" ]; then
    echo "Error: cerficifateFile "${certificateFilePath}" not exists"
    exit
fi

if az iot hub list | jq -r .[].name | grep "${iotHubName}" >/dev/null 2>&1; then
    echo "Error: iotHubName ${iotHubName} already exists"
    exit
fi

if az iot dps list | jq -r .[].name | grep "${provisioningServiceName}" >/dev/null 2>&1; then
    echo "Error: provisioningServiceName ${provisioningServiceName} already exists"
    exit
fi

function delete_resource() {
    case $1 in
        "hub" )
            echo "Delete IoT Hub."
            az iot hub delete -g "${resourceGroup}" -n "${iotHubName}"
            ;;
        "dps" )
            echo "Delete DPS."
            az iot dps delete -g "${resourceGroup}" -n "${provisioningServiceName}"
            ;;
        "all" )
            echo "Delete IoT Hub and DPS."
            az iot hub delete -g "${resourceGroup}" -n "${iotHubName}"
            az iot dps delete -g "${resourceGroup}" -n "${provisioningServiceName}"
            ;;
    esac
}

echo "Starting to create IoT Hub."
if ! az iot hub create -g "${resourceGroup}" -n "${iotHubName}" --sku "${skuName}" --unit "${skuUnit}" --partition-count "${partitionCount}"; then
    echo "Error: create IoT Hub."
    exit
fi

echo "Starting to create DPS. "
if ! az iot dps create -g "${resourceGroup}" -n "${provisioningServiceName}"; then
    echo "Error: create DPS."
    delete_resource "hub"
    exit
fi

echo "Starting to link between IoT Hub and DPS."
connectionString=$(az iot hub connection-string show -n "${iotHubName}" --query connectionString | sed -r 's/^"(.*)"$/\1/')
if ! az iot dps linked-hub create --dps-name "${provisioningServiceName}" --connection-string "${connectionString}"; then
    echo "Error: link between IoT Hub and DPS."
    delete_resource "all"
    exit
fi

echo "Starting to create enrollment."
deviceId=$(openssl x509 -noout -subject -in "${certificateFilePath}" | grep subject | awk '{print $NF}')
if ! az iot dps enrollment create --dps-name "${provisioningServiceName}" --eid "${deviceId}" --at x509 -d "${deviceId}" --cp "${certificateFilePath}"; then
    echo "Error: create enrollment."
    delete_resource "all"
    exit
fi

echo "Completed!"
